{

Simple 1-character demo

This example uses an 8x8 character, 2 color (foreground/background) text driver
to draw a 16x16 map on the screen and a simple 8x16 character that can move around
using the keyboard/gamepad.

A similar design could be accomplished with other Propeller/Hydra display drivers.

If you look at the Font_LinkDemo.spin and accompanying Font_LinkDemo.bmp, you will be
able to see the 'font' definition used in the demo.  Font_LinkDemo.spin was generated
with the FontGenerator utility (see: http://forums.parallax.com/forums/default.aspx?f=33&m=377666),
using the P (Potato_Text) option.

The DAT section (at the bottom of the listing) contains the map data used in the game.

In the VAR section, you will see that the character is managed with 4 variables (direction,
animation step, and x/y location).

The actual game loop does the following:
  1) Draw the map (background)          
  2) Check for gamepad events
  3) Draw the character 
  4) Update the display with changes

 
}

CON
  ' Proto/Demo Board setup
   _clkmode  = xtal1 + pll16x
   _xinfreq  = 5_000_000

   TV_BASE_PIN = 12
   KEYSPIN   = 26

{   
  ' Hydra setup
  _CLKMODE = xtal1 + pll8x
  _XINFREQ = 10_000_000 + 0000

   TV_BASE_PIN = 24 
   KEYSPIN  = 13   
}

{
' Hybrid Boards
   _clkmode  = xtal1 + pll16x
   _xinfreq  = 6_000_000
   
   TV_BASE_PIN = 24    
   KEYSPIN  = 12 
}

'' gamepad pin configuration (adjust for your setup)
  JOY_CLK = 3
  JOY_LCH = 4
  JOY_DATAOUT0 = 5
  JOY_DATAOUT1 = 6 
  
  ' NES bit encodings for NES gamepad 0
  NES0_RIGHT    = %00000000_00000001
  NES0_LEFT     = %00000000_00000010
  NES0_DOWN     = %00000000_00000100
  NES0_UP       = %00000000_00001000
  NES0_START    = %00000000_00010000
  NES0_SELECT   = %00000000_00100000
  NES0_B        = %00000000_01000000
  NES0_A        = %00000000_10000000  

  COLOR_BLACK  = $02
  COLOR_BROWN  = $5d
  COLOR_YELLOW = $6e
  COLOR_WHITE  = $07
  COLOR_GREEN  = $9e
  COLOR_RED    = $4b
  COLOR_PURPLE = $2b
  COLOR_CYAN   = $58
  COLOR_BLUE   = $68
  

  'direction constants
  DIR_SOUTH = 0
  DIR_NORTH = 2
  DIR_WEST  = 4
  DIR_EAST  = 6

VAR
  word Nes_Pad

  byte player_dir      'the player direction
  byte player_step     'the step the player is on
  byte player_x        'x position
  byte player_y        'y position

  byte baddie1_x
  byte baddie1_y
  byte baddie2_x
  byte baddie2_y
  byte baddie3_x
  byte baddie3_y
  byte baddie4_x
  byte baddie4_y

  byte blueghosttimer 'Simple tick timer for blue ghosts
  byte ghosttime      'Total ticks that ghosts are blue

  byte pellets        'pellet counter
  
OBJ
  text : "LinkDemo_TV_Text_004"  'the TV display driver
  key  : "comboKeyboard"

  pst    : "Parallax Serial Terminal" 
  
PUB start 

    pst.Start(115200)

    'start the tv text driver
    text.start(TV_BASE_PIN) 
    key.start(KEYSPIN)

    'set up the initial character location/facing
    player_dir  := DIR_EAST
    player_step := 0
    player_x    := 16
    player_y    := 10

    baddie1_x   :=10
    baddie1_y   :=10
   
    baddie2_x   :=10
    baddie2_y   :=11
   
    baddie3_x   :=10
    baddie3_y   :=9

    baddie4_x   :=9
    baddie4_y   :=10

    blueghosttimer  := 101
    ghosttime       := 100
    pellets         := 0
    
    'set up the game loop
    repeat
      'Draw the map (background)          
      DrawMap
      
      'Check for gamepad events
      CheckGamePad

      'draw the player 
      DrawPlayer

      'draw baddies
      DrawBaddie1(blueghosttimer)
      DrawBaddie2(blueghosttimer)
      DrawBaddie3(blueghosttimer)
      DrawBaddie4(blueghosttimer)
      
      'build in a delay for animation
      waitcnt(5_000_000 + cnt)

      'update the screen with changes
      text.UpdateScreen

       'Tick the blueghosttimer
      blueghosttimer++
      if blueghosttimer == 254
         blueghosttimer := 101

      'Check to see if map is finished!
      if pellets == 151
            player_dir  := DIR_EAST
            player_step := 0
            player_x := 16
            player_y := 10

            baddie1_x :=10
            baddie1_y :=10
   
            baddie2_x :=10
            baddie2_y :=11
   
            baddie3_x :=10
            baddie3_y :=9

            baddie4_x :=9
            baddie4_y :=10

            blueghosttimer := 101
            ghosttime := 100
            pellets := 0
   
      
PRI DrawMap | ptr, x, y
    ptr := @mapinfo
    repeat x from 1 to 21
       repeat y from 1 to 19 'width

          'determine the color based on the character:
          ' $20 = empty space
          ' $2E (".")= pellet
          ' $60 to $7F = maze walls
          ' $2F ("/") power pellet
          case byte[ptr]
             $20      : text.PokeChar(x,y,COLOR_YELLOW,COLOR_BLACK, byte[ptr])
             $2E      : text.PokeChar(x,y,COLOR_WHITE,COLOR_BLACK, byte[ptr])
             $2F      : text.PokeChar(x,y,COLOR_WHITE,COLOR_BLACK, byte[ptr])
             $60..$7F : text.PokeChar(x,y,COLOR_BROWN,COLOR_BLACK, byte[ptr])         
          ptr++
  
PRI CheckGamePad
     Nes_Pad := NES_Read_Gamepad
     'gamepad has been moved down (south)               
     if (Nes_Pad & NES0_DOWN)
        player_dir := DIR_SOUTH
        
        'check to make sure the space the character is moving into
        'is clear
        if (CheckMove(player_x+1,player_y) == 0)
           player_x++
           'bounds check: if the player_x is > 21, they would be 'out of bounds'
           if (player_x > 21)
              player_x := 21
           PlayerMove         

     'gamepad has been moved up (north)                
     if (Nes_Pad & NES0_UP)
        player_dir := DIR_NORTH
        
        'check to make sure the space the character is moving into
        'is clear
        if (CheckMove(player_x-1,player_y) == 0)
           player_x--
           'bounds check: if the player_x is < 1, they would be 'out of bounds'           
           if (player_x < 1)
              player_x := 1
           PlayerMove 

     'gamepad has been moved left (west)           
     if (Nes_Pad & NES0_LEFT)
        player_dir := DIR_WEST

        'check to make sure the space the character is moving into
        'is clear        
        if (CheckMove(player_x,player_y-1) == 0)
           player_y--
           'bounds check: if the player_y is < 1, they would be 'out of bounds'
           if (player_y < 1)
              player_y := 1
           PlayerMove

     'gamepad has been moved right (east)                        
     if (Nes_Pad & NES0_RIGHT)
        player_dir := DIR_EAST

        'check to make sure the space the character is moving into
        'is clear        
        if (CheckMove(player_x,player_y+1) == 0)
           player_y++
           'bounds check: if the player_y is > 19, they would be 'out of bounds'
           if (player_y > 19)
              player_y := 19           
           PlayerMove          

PRI CheckMove (move_x, move_y) | blocked
    blocked := true

    'test the specific character
    case text.PeekChar(move_x , move_y)
      'Empty Space
      $20      : blocked := false

      'Pellet
      $2E      : blocked := false

                MunchPellet(move_x, move_y)

      'Power Pellet
      $2F      : blocked := false
                'since we know we have 'hit' a pellet,
                'we can set the position to 'space' and
                'add to the score
                 MunchPellet(move_x, move_y)
                 blueghosttimer:=0

      'hit a wall
      $60..$7F : blocked := true    
    return blocked

PRI MunchPellet (x, y) | ptr
   ptr := @mapinfo
   
   'find the map position in the map information
   '(we subtract 1 from the x and the y, since
   'the data is 0-based
   ptr += ((x-1) * 19) + (y-1)
   
   'set the map info to space character
   byte[ptr] := $20

   pellets++

         
PRI PlayerMove
    'advance the player step, and reset to 0 when necessary
    player_step++
    if (player_step > 1)
       player_step := 0
                      
PRI DrawPlayer | offset
    offset := player_dir + player_step
    text.PokeChar(player_x , player_y, COLOR_YELLOW,COLOR_BLACK,$02 + offset)

PRI DrawBaddie1(timer) | offset

    if timer > ghosttime
    'offset := player_dir + player_step
       text.PokeChar(baddie1_x , baddie1_y, COLOR_BROWN,COLOR_BLACK,$2D)
       
    if timer < ghosttime
       text.PokeChar(baddie1_x , baddie1_y, COLOR_BLUE,COLOR_BLACK,$2D)
       
PRI DrawBaddie2(timer) | offset

    if timer > ghosttime
    'offset := player_dir + player_step
        text.PokeChar(baddie2_x , baddie2_y, COLOR_YELLOW,COLOR_BLACK,$2D)
    if timer < ghosttime

        text.PokeChar(baddie2_x , baddie2_y, COLOR_BLUE,COLOR_BLACK,$2D)
PRI DrawBaddie3(timer) | offset

    if timer > ghosttime

    'offset := player_dir + player_step
       text.PokeChar(baddie3_x , baddie3_y, COLOR_CYAN,COLOR_BLACK,$2D)

    if timer < ghosttime
        text.PokeChar(baddie3_x , baddie3_y, COLOR_BLUE,COLOR_BLACK,$2D)
    
PRI DrawBaddie4(timer) | offset
   
    if timer  > ghosttime
    'offset := player_dir + player_step
       text.PokeChar(baddie4_x , baddie4_y, COLOR_RED,COLOR_BLACK,$2D)
    
    if timer < ghosttime
       text.PokeChar(baddie4_x , baddie4_y, COLOR_BLUE,COLOR_BLACK,$2D)
    
PRI NES_Read_Gamepad : nes_bits        |  i
' //////////////////////////////////////////////////////////////////
' NES Game Paddle Read
' //////////////////////////////////////////////////////////////////       
' reads both gamepads in parallel encodes 8-bits for each in format
' right game pad #1 [15..8] : left game pad #0 [7..0]
'
' set I/O ports to proper direction
' P3 = JOY_CLK      (4)
' P4 = JOY_SH/LDn   (5) 
' P5 = JOY_DATAOUT0 (6)
' P6 = JOY_DATAOUT1 (7)
' NES Bit Encoding
'
' RIGHT  = %00000001
' LEFT   = %00000010
' DOWN   = %00000100
' UP     = %00001000
' START  = %00010000
' SELECT = %00100000
' B      = %01000000
' A      = %10000000

' step 1: set I/Os
DIRA [JOY_CLK] := 1 ' output
DIRA [JOY_LCH] := 1 ' output
DIRA [JOY_DATAOUT0] := 0 ' input
DIRA [JOY_DATAOUT1] := 0 ' input

' step 2: set clock and latch to 0
OUTA [JOY_CLK] := 0 ' JOY_CLK = 0
OUTA [JOY_LCH] := 0 ' JOY_SH/LDn = 0
'Delay(1)

' step 3: set latch to 1
OUTA [JOY_LCH] := 1 ' JOY_SH/LDn = 1
'Delay(1)                            

' step 4: set latch to 0
OUTA [JOY_LCH] := 0 ' JOY_SH/LDn = 0

' data is now ready to shift out, clear storage
nes_bits := 0

' step 5: read 8 bits, 1st bits are already latched and ready, simply save and clock remaining bits
repeat i from 0 to 7

 nes_bits := (nes_bits << 1)
 nes_bits := nes_bits | INA[JOY_DATAOUT0] | (INA[JOY_DATAOUT1] << 8)

 OUTA [JOY_CLK] := 1 ' JOY_CLK = 1
 'Delay(1)             
 OUTA [JOY_CLK] := 0 ' JOY_CLK = 0
 
 'Delay(1)             
' invert bits to make positive logic
nes_bits := (!nes_bits & $FFFF)

' Keyboard (mapped onto NES buttons)
    if(key.keystate($C2))       'Up Arrow
      nes_bits|=NES0_UP
    if(key.keystate($C3))       'Down Arrow
      nes_bits|=NES0_DOWN
    if(key.keystate($C0))       'Left Arrow
      nes_bits|=NES0_LEFT
    if(key.keystate($C1))       'Right Arrow
      nes_bits|=NES0_RIGHT
    if(key.keystate($0D))       'Enter
      nes_bits|=NES0_START
    if(key.keystate(" "))       'Space
      nes_bits|=NES0_B

' //////////////////////////////////////////////////////////////////
' End NES Game Paddle Read
' //////////////////////////////////////////////////////////////////             
DAT
'Map information: d = rock / {space} = open

mapinfo byte "hllllslllllllsllllj"
        byte "i....i.......i....i"  
        byte "i/on.q ollln.q on/i"  
        byte "i.................i" 
        byte "kn.p oln p.oln p.om" 
        byte "  .i.....i.....i.  "  
        byte "oj.kln olrln olm.hn" 
        byte " i....       ....i "  
        byte " i.hln hltlj olj.i "  
        byte " i.i.. i   i ..i.i " 
        byte "om.q p klllm p.q.kn" 
        byte "  ...i       i...  "
        byte "oj.olrln p olrln.hn"
        byte " i.......i.......i "
        byte "hm.oln.olrln.oln.kj"
        byte "i........ ........i"  
        byte "i.hlj.hn.p.oj.hlj.i"  
        byte "i/i i.i..i..i.i i/i"  
        byte "i.klm.q.orn.q.klm.i"  
        byte "i.................i" 
        byte "klllllllllllllllllm"

      